#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

import construct

from base_msg import PSBP, SOURCE_ADDRESS
from utils import fmt_repr, containerize, exclude_fields

PSBP_MSG_I_MAX_W_REQ = 0x13


class MsgImaxWReq(PSBP):
    """
    power W req
    """
    _parser = construct.Struct(
        'set_value' / construct.Int16ub)
    __slots__ = [
        'set_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgImaxWReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            self.from_binary(psbp.payload)
        else:
            super(MsgImaxWReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_I_MAX_W_REQ
            self.set_value = kwargs.pop('set_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgImaxWReq._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgImaxWReq._parser.build(c)
        return self.pack()


PSBP_MSG_I_MAX_W_ACK = 0x13


class MsgImaxWAck(PSBP):
    """
    power W ack
    """
    _parser = construct.Struct(
        'actual_value' / construct.Int16ub)
    __slots__ = [
        'actual_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgImaxWAck,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgImaxWAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_I_MAX_W_ACK
            self.actual_value = kwargs.pop('actual_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgImaxWAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgImaxWAck._parser.build(c)
        return self.pack()


PSBP_MSG_I_MAX_R_REQ = 0x12


class MsgImaxRReq(PSBP):
    """
    power R req
    """

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgImaxRReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            super.from_binary(psbp.payload)
        else:
            super(MsgImaxRReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_I_MAX_R_REQ

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = super(MsgImaxRReq, self)._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))


PSBP_MSG_I_MAX_R_ACK = 0x12


class MsgImaxRAck(PSBP):
    """
    power R ack
    """
    _parser = construct.Struct(
        'actual_value' / construct.Int16ub)
    __slots__ = [
        'actual_value',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgImaxRAck, self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                              psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgImaxRAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_I_MAX_R_ACK
            self.actual_value = kwargs.pop('actual_value')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgImaxRAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgImaxRAck._parser.build(c)
        return self.pack()


msg_classes = {
    0x13: MsgImaxWAck,
    0x12: MsgImaxRAck,
}
