#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

import construct

from base_msg import PSBP, SOURCE_ADDRESS
from utils import fmt_repr, containerize, exclude_fields

PSBP_MSG_MODE_W_REQ = 0x15


class MsgModeWReq(PSBP):
    """
    mode W req
    """
    _parser = construct.Struct(
        'mode' / construct.Int8ub)
    __slots__ = [
        'mode',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgModeWReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            self.from_binary(psbp.payload)
        else:
            super(MsgModeWReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_MODE_W_REQ
            self.mode = kwargs.pop('mode')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgModeWReq._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgModeWReq._parser.build(c)
        return self.pack()


PSBP_MSG_MODE_W_ACK = 0x15


class MsgModeWAck(PSBP):
    """
    mode W ack
    """
    _parser = construct.Struct(
        'mode' / construct.Int8ub)
    __slots__ = [
        'mode',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgModeWAck,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgModeWAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_MODE_W_ACK
            self.mode = kwargs.pop('mode')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgModeWAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgModeWAck._parser.build(c)
        return self.pack()


PSBP_MSG_MODE_R_REQ = 0x14


class MsgModeRReq(PSBP):
    """
    mode R req
    """

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgModeRReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            super.from_binary(psbp.payload)
        else:
            super(MsgModeRReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_MODE_R_REQ

    def __repr__(self):
        return fmt_repr(self)


PSBP_MSG_MODE_R_ACK = 0x14


class MsgModeRAck(PSBP):
    """
    mode R ack
    """
    _parser = construct.Struct(
        'mode' / construct.Int8ub)
    __slots__ = [
        'mode',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgModeRAck, self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                              psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgModeRAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_MODE_R_ACK
            self.mode = kwargs.pop('mode')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgModeRAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgModeRAck._parser.build(c)
        return self.pack()


msg_classes = {
    0x15: MsgModeWAck,
    0x14: MsgModeRAck,
}

if __name__ == '__main__':
    print('Simple Test for a Mode Messages')
    msg = MsgModeWReq(receiver_address=0x40, flag=(1 << 4), mode=100)
    packed_msg = msg.pack()
    print(':'.join(hex(ord(x))[2:] for x in packed_msg))
    raw_bin = msg.to_binary()
    print(':'.join(hex(ord(x))[2:] for x in raw_bin))

    msg = MsgModeRReq(receiver_address=0x40, flag=(1 << 4))
    packed_msg = msg.pack()
    print(':'.join(hex(ord(x))[2:] for x in packed_msg))
    raw_bin = msg.to_binary()
    print(':'.join(hex(ord(x))[2:] for x in raw_bin))
